/' Include: bglBase.bi '/
#pragma Once

#include Once "../Utils/fbUtils.bi"
#include Once "../Utils/fbEvents.bi"
#include Once "OpenGL.bi"

Const _
  DefaultWidth  = 640, _
  DefaultHeight = 480, _
  DefaultTitle  = "FB - OpenGL BASE :: www.qbasic.de/forum"

Type _
  TGLBase Extends THandleEvents
    Public:
      Declare Constructor ()
      Declare Virtual Sub Create(ByVal Ww As Integer= DefaultWidth, _
                                 ByVal Wh As Integer= DefaultHeight, _
                                 ByRef WTitle As Const String= DefaultTitle, _
                                 ByVal FullScreen As Boolean= False)
      Declare Virtual Sub HideMouse ()
      Declare Virtual Sub ShowMouse ()

      Declare Virtual Destructor ()
      
      Declare Property ScreenWidth As Integer '' read vScreenWidth
      Declare Property ScreenHeight As Integer '' read vScreenHeight
    Protected:
      Declare Sub UserProc () Override
      Declare Virtual Sub UpdateWorld(ByVal MSec As GLfloat)
      Declare Abstract Sub DrawScene ()
      Declare Abstract Sub InitOpenGL ()
      
      Declare Sub Render () /' wird nur verwendet um die FPS an zu zeigen,
                                kann also auch weg gelassen werden. '/
    Private:
      Title        As ZString Ptr
      ColorDepth   As Integer
      vScreenWidth  As Integer
      vScreenHeight As Integer
      ScreenFlag   As Integer
      
      StartTime    As Double
      LastUpdate   As Double
      
      Frames       As Integer
      FPS          As Integer
  End Type

'' --- TGLBase -----------------------------------------------------
''                              PUBLIC
'' -------------------------------------------------------------------

'' --- TGLBase -----------------------------------------------------
''  Constructor
'' -------------------------------------------------------------------
Constructor TGLBase ()
  ScreenInfo(,, ColorDepth)
  ScreenFlag= FB.GFX_OPENGL Or FB.GFX_NO_SWITCH
  Title= Nil
End Constructor

'' --- TGLBase -----------------------------------------------------
''  Create
'' -------------------------------------------------------------------
Sub TGLBase.Create(ByVal Ww As Integer= DefaultWidth, _
                      ByVal Wh As Integer= DefaultHeight, _
                      ByRef WTitle As Const String= DefaultTitle, _
                      ByVal FullScreen As Boolean= False)
  If FullScreen Then
    ScreenInfo(vScreenWidth, vScreenHeight)
    ScreenFlag= ScreenFlag Or FB.GFX_FULLSCREEN
  Else
    vScreenWidth= Ww
    vScreenHeight= Wh
    Title= Callocate (Len (WTitle)+ 1)
    *Title= WTitle
  End If

  ScreenRes(vScreenWidth, vScreenHeight, ColorDepth,, ScreenFlag)
  If(Title <> Nil) Then
    WindowTitle(*Title)
  End If
  
  InitOpenGL ()
  
  Frames= 0
  StartTime= Timer
  LastUpdate= Timer
End Sub

'' --- TGLBase -----------------------------------------------------
''  HideMouse
'' -------------------------------------------------------------------
Sub TGLBase.HideMouse ()
  SetMouse (,, 0)
End Sub

'' --- TGLBase -----------------------------------------------------
''  ShowMouse
'' -------------------------------------------------------------------
Sub TGLBase.ShowMouse ()
  SetMouse (,, 1)
End Sub

'' --- TGLBase -----------------------------------------------------
''  Destructor
'' -------------------------------------------------------------------
Destructor TGLBase ()
  Screen(0)             '' kann man auch weg lassen
  DeAllocate (Title)
End Destructor

Property TGLBase.ScreenWidth As Integer '' read vScreenWidth
  Return (vScreenWidth)
End Property

Property TGLBase.ScreenHeight As Integer '' read vScreenHeight
  Return (vScreenHeight)
End Property

'' --- TGLBase -----------------------------------------------------
''                              PROTECTED
'' -------------------------------------------------------------------

'' --- TGLBase -----------------------------------------------------
''  IdleProc
'' -------------------------------------------------------------------
Sub TGLBase.UserProc ()  
Dim TimeNow As Double

  TimeNow= Timer
  If(TimeNow > LastUpdate) Then
    UpdateWorld(TimeNow- LastUpdate)
    LastUpdate= TimeNow
  End If
  DrawScene ()
  Render ()
End Sub

'' --- TGLBase -----------------------------------------------------
''  UpdateWorld
'' -------------------------------------------------------------------
Sub TGLBase.UpdateWorld(ByVal MSec As GLfloat)
  '' muss vom nachfolger Definiert werden
End Sub

'' --- TGLBase -----------------------------------------------------
''  Render
'' -------------------------------------------------------------------
Sub TGLBase.Render ()
  Frames= Frames+ 1
  If((Timer- StartTime) >= 0.500) Then
    FPS= Frames/(Timer- StartTime)
    Frames= 0
    StartTime= Timer
    
    If(Title <> Nil) Then
      WindowTitle(*Title+ " - "+ Str(FPS)+ " FPS")
    End If
  End If

  Flip ()
End Sub

'' --- TGLBase -----------------------------------------------------
''                              PRIVATE
'' -------------------------------------------------------------------
