/' Include: bglTextures.bi '/
#pragma Once

#include Once "../Utils/fbUtils.bi"
#include Once "../Utils/LoadImage.bi"
#include Once "OpenGL.bi"
#include Once "crt/string.bi"

Type TTextureAlias As String
Type TTexture As GLuInt


Type PTextureInfo As TTextureInfo Ptr
Type _
  TTextureInfo Field= 1
    TexAlias As String
    TexID As TTexture
    RGBLen As GLint
    RGBFormat As GLenum
    minFilter As GLenum
    magFilter As GLenum
    Mipmap As Boolean
    TexWidth As Integer
    TexHeight As Integer
    Buffer As uByte Ptr
  End Type

Type _
  TGLTextures Extends TFBImage
    Private :      
      TexList(Any) As TTextureInfo
      Declare Sub SwapRB (ByVal Image As PImage)
      Declare Function CreateTexture (ByRef Texture As TTextureInfo) As Integer
      Declare Function ExistIndex (ByVal Index As Integer) As Boolean
    Public :
      Declare Constructor ()
      
      /' Die Filter und Mipmap-Auswertung ist (etwas abgewandelt) aus den
         NeHe-Examples - FreeBASIC->Examples->Graphics->OpenGL->NeHe - createtex.bi
         entnommen! '/
      Declare Function CreateTextureFromFile (ByRef FileName As Const String, _
                                              ByVal TexAlias As String, _
                                              ByVal ClearImage As Boolean= True, _
                                              ByVal Filter As Boolean= False, _
                                              ByVal Mipmap As Boolean= False) As Integer
      Declare Sub CreateTextureFromMemory Overload (ByVal TexAlias As String)
      Declare Sub CreateTextureFromMemory (ByVal Index As Integer)
      Declare Function GetInfo (ByRef Index As Integer) As TTextureInfo
      Declare Sub ClearTexture (ByVal Index As Integer)
      Declare Virtual Destructor ()
      
      Declare Property GetIndex (ByVal TexAlias As String) As Integer '' read TexAlias.IndexOf ()
      Declare Property GetID (ByVal Index As Integer) As TTexture
      Declare Property GetWidth (ByVal Index As Integer) As Integer
      Declare Property GetHeight (ByVal Index As Integer) As Integer
  End Type

'' ########################################################################
''  TGLTextures Objekt
'' ########################################################################
Private Constructor TGLTextures ()
End Constructor

Private Function TGLTextures.CreateTextureFromFile (ByRef FileName As Const String, _
                                                    ByVal TexAlias As String, _
                                                    ByVal ClearImage As Boolean= True, _
                                                    ByVal Filter As Boolean= False, _
                                                    ByVal Mipmap As Boolean= False) As Integer
Dim Image As PImage= Nil
Dim Texture As TTexture= 0
Dim Result As Integer
Dim As GLEnum TexFormat, MinFilter, MagFilter
Dim SystemFormat As Integer
  ScreenInfo (,,, SystemFormat)

    Image= LoadFromFile (FileName)
    If(Image <> Nil) Then
      If(FBI_Bpp (Image) = 4) Then
        TexFormat= GL_RGBA
      Else
        TexFormat= GL_RGB
      End If
    
      FlipHorizontal (Image)
      SwapRB (Image)
    
      If Not Filter Then
        MagFilter= GL_NEAREST
      Else
        Magfilter= GL_LINEAR
      End If

      If Mipmap Then
        If Not Filter Then
          MinFilter= GL_LINEAR_MIPMAP_NEAREST
        Else
          MinFilter= GL_LINEAR_MIPMAP_LINEAR
        End If
      Else
        MinFilter= MagFilter
      End If
      
      If(uBound (TexList) >= lBound (TexList)) Then
        ReDim Preserve TexList(uBound (TexList)+ 1)
      Else
        ReDim TexList(uBound (TexList)+ 1)
      End If
      
      TexList(uBound (TexList)).TexAlias= TexAlias
      TexList(uBound (TexList)).TexID= 0
      TexList(uBound (TexList)).RGBLen= SystemFormat
      TexList(uBound (TexList)).RGBFormat= TexFormat
      TexList(uBound (TexList)).minFilter= MinFilter
      TexList(uBound (TexList)).magFilter= MagFilter
      TexList(uBound (TexList)).Mipmap= Mipmap
      TexList(uBound (TexList)).TexWidth= FBI_Width (Image)
      TexList(uBound (TexList)).TexHeight= FBI_Height (Image)
      TexList(uBound (TexList)).Buffer= CAllocate (FBI_Size (Image)- SizeOf (TImage))
      MemCpy (TexList(uBound (TexList)).Buffer, FBI_Pixels (Image), FBI_Size (Image)- SizeOf (TImage))
      
      Result= CreateTexture (TexList(uBound (TexList)))
      ImageDestroy (Image)
      
      If ClearImage Then
        DeAllocate (TexList(uBound (TexList)).Buffer)
        TexList(uBound (TexList)).Buffer= Nil
      End If
    End If

  Function= Result
End Function

Private Sub TGLTextures.CreateTextureFromMemory Overload (ByVal TexAlias As String)
Dim Index As Integer

  Index= GetIndex (TexAlias)
  If(Index > -1) Then
    CreateTexture (TexList(Index))
  End If
End Sub

Private Sub TGLTextures.CreateTextureFromMemory (ByVal Index As Integer)
  If ExistIndex (Index) Then
    CreateTexture (TexList(Index))
  End If
End Sub

Private Function TGLTextures.GetInfo (ByRef Index As Integer) As TTextureInfo
Dim Result As TTextureInfo

  If ExistIndex (Index) Then
    Result= TexList(Index)
  End If
  
  Return (Result)
End Function

Private Sub TGLTextures.ClearTexture (ByVal Index As Integer)
  If ExistIndex (Index) Then
    glDeleteTextures (1, @TexList(Index).TexID)
    TexList(Index).TexID= 0
  End If
End Sub

Private Destructor TGLTextures ()

  If(uBound (TexList) >= lBound (TexList)) Then
    For I As Integer= lBound (TexList) To uBound (TexList)
      If(TexList(I).TexID > 0) Then
        glDeleteTextures (1, @TexList(I).TexID)
      End If
      If(TexList(I).Buffer <> Nil) Then
        DeAllocate (TexList(I).Buffer)
      End If
    Next I
    
    Erase (TexList)
  End If

End Destructor

Private Property TGLTextures.GetIndex (ByVal TexAlias As String) As Integer
Dim Index As Integer= -1

  If(uBound (TexList) >= lBound (TexList)) Then
    For I As Integer= lBound (TexList) To uBound (TexList)
      If(TexList(I).TexAlias = TexAlias) Then
        Index= I
        Exit For
      End If
    Next I
  End If
  
  Property= Index
End Property

Private Property TGLTextures.GetID (ByVal Index As Integer) As TTexture
Dim Result As TTexture= 0

  If ExistIndex (Index) Then
    Result= TexList(Index).TexID
  End If
  
  Property= Result
End Property

Private Property TGLTextures.GetWidth (ByVal Index As Integer) As Integer
Dim Result As Integer= 0

  If ExistIndex (Index) Then
    Result= TexList(Index).TexWidth
  End If
  
  Property= Result
End Property

Private Property TGLTextures.GetHeight (ByVal Index As Integer) As Integer
Dim Result As Integer= 0

  If ExistIndex (Index) Then
    Result= TexList(Index).TexHeight
  End If
  
  Property= Result
End Property

/' Private '/
Private Sub TGLTextures.SwapRB (ByVal Image As PImage)
Dim As uByte Ptr Src, Buf
Dim As uByte Tmp
Dim As Integer Bpp

  Bpp= FBI_Bpp (Image)
  Buf= Allocate (Bpp)
  Src= FBI_Pixels (Image)

  For I As Integer= 1 To FBI_Width (Image)* FBI_Height (Image)
    'Swap Src[0], Src[2]
    
    /' Ist ein hauch schneller! '/
    Tmp= Src[0]
    Src[0]= Src[2]
    Src[2]= Tmp
    Src= Src+ Bpp
  Next I

  DeAllocate (Buf)
End Sub

Private Function TGLTextures.CreateTexture (ByRef Texture As TTextureInfo) As Integer
  glGenTextures (1, @Texture.TexID)
  glBindTexture (GL_TEXTURE_2D, Texture.TexID)

  If Texture.Mipmap Then
    gluBuild2DMipmaps (GL_TEXTURE_2D, Texture.RGBLen, Texture.TexWidth, Texture.TexHeight, _
                       Texture.RGBFormat, GL_UNSIGNED_BYTE, Texture.Buffer)
  Else
    glTexImage2D (GL_TEXTURE_2D, 0, Texture.RGBLen, Texture.TexWidth, Texture.TexHeight, _
                  0, Texture.RGBFormat, GL_UNSIGNED_BYTE, Texture.Buffer)
  End If
  
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, Texture.minFilter)
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, Texture.magFilter)
    
  Return (uBound (TexList))
End Function

Private Function TGLTextures.ExistIndex (ByVal Index As Integer) As Boolean
Dim Result As Boolean= False

  If(uBound (TexList) >= lBound (TexList)) Then
    If(Index >= lBound (TexList)) And (Index <= uBound (TexList)) Then
      Result= True
    End If
  End If
  
  Function= Result
End Function

