/' Include: LoadImage.bi '/
#pragma Once

#include Once "fbgfx.bi"
#include Once "fbUtils.bi"
#include Once "FBImage.bi"
#include Once "crt/string.bi" /' MemCpy () '/

Type PImage As TImage Ptr
Type TImage As FB.Image

Type _
  TFBImage Extends Object
    Public :
      Declare Constructor () /' default '/
      Declare Virtual Function LoadFromFile (ByRef FileName As Const String) As PImage
      Declare Sub FlipHorizontal (ByRef Image As PImage)
      Declare Sub FlipVertical (ByVal Image As PImage)
      Declare Sub RotateLeft (ByRef Image As PImage)
      Declare Sub RotateRight (ByRef Image As PImage)
      
      Declare Property FBI_Width (ByVal Image As PImage) As Integer    ' read ImageInfo (, Breite)
      Declare Property FBI_Height (ByVal Image As PImage) As Integer   ' read   ""      (,, Hoehe)
      Declare Property FBI_Bpp (ByVal Image As PImage) As Integer      ' read   ""      (,,, Bpp)
      Declare Property FBI_Pitch (ByVal Image As PImage) As Integer    ' read   ""      (,,,, Pitch)
      Declare Property FBI_Pixels (ByVal Image As PImage) As uByte Ptr ' read   ""      (,,,,, Pixels)
      Declare Property FBI_Size (ByVal Image As PImage) As Integer     ' read   ""      (,,,,,, Size)
  End Type

/' --- TFBImage object -------------------------------------------- '/
Private Constructor TFBImage ()
End Constructor

Private Function TFBImage.LoadFromFile (ByRef FileName As Const String) As PImage
Dim Image As PImage

  If FileExist (FileName) Then
    Image= LoadRGBAFile (FileName)
    If(Image = Nil) Then
      ConsoleMsg ("Fehler: "& *GetLastResult ())
    End If
  Else
    Image= Nil
  End If
  
  Function= Image
End Function

Private Sub TFBImage.FlipHorizontal (ByRef Image As PImage)
Dim Buffer As PImage
Dim As uByte Ptr Src, Dst
Dim Pitch As Integer

  Buffer= ImageCreate (FBI_Width (Image), FBI_Height (Image), 0, FBI_Bpp (Image)* 8)
  Pitch= FBI_Pitch (Image)
  Src= FBI_Pixels (Image)
  Dst= FBI_Pixels (Buffer)+ (FBI_Height (Buffer)- 1)* Pitch
  For Z As Integer= 1 To FBI_Height (Image)
    MemCpy (Dst, Src, Pitch)
    Src= Src+ Pitch
    Dst= Dst- Pitch
  Next Z
  
  ImageDestroy (Image)
  Image= Buffer
End Sub

Private Sub TFBImage.FlipVertical (ByVal Image As PImage)
Dim as uByte Ptr Start, Src, Dst, Buf
Dim As Integer Pitch, Bpp

  Bpp= FBI_Bpp (Image)
  Buf= Allocate (Bpp)
  Pitch= FBI_Pitch (Image)
  Start= FBI_Pixels (Image)
  
  For Y As Integer= 1 To FBI_Height (Image)
    Src= Start
    Dst= Src+ (FBI_Width (Image))* Bpp
    For X As Integer= 1 To FBI_Width (Image)\ 2
      MemCpy (Buf, Src, Bpp)
      MemCpy (Src, Dst, Bpp)
      MemCpy (Dst, Buf, Bpp)
      
      Src= Src+ Bpp
      Dst= Dst- Bpp
    Next X
    Start= Start+ Pitch
  Next Y
  
  DeAllocate (Buf)
End Sub

Private Sub TFBImage.RotateLeft (ByRef Image As PImage)
Dim Buffer As PImage
Dim As uByte Ptr Src, Dst, Buf, Start
Dim As Integer Bpp, Pitch

  Bpp= FBI_Bpp (Image)
  Buffer= ImageCreate (FBI_Height (Image), FBI_Width (Image), 0, Bpp* 8)
  Buf= Allocate (Bpp)
  Pitch= FBI_Pitch (Image)
  Start= FBI_Pixels (Image)+ Pitch- Bpp
  Dst= FBI_Pixels (Buffer)
  For Y As Integer= 1 To FBI_Height (Buffer)
    Src= Start
    For X As Integer= 1 To FBI_Width (Buffer)
      MemCpy (Dst, Src, Bpp)
      Dst= Dst+ Bpp
      Src= Src+ Pitch
    Next X
    Start= Start- Bpp
  Next Y
  
  ImageDestroy (Image)
  Image= Buffer

  DeAllocate (Buf)
End Sub

Private Sub TFBImage.RotateRight (ByRef Image As PImage)
Dim Buffer As PImage
Dim As uByte Ptr Src, Dst, Buf, Start
Dim As Integer Bpp, Pitch

  Bpp= FBI_Bpp (Image)
  Buffer= ImageCreate (FBI_Height (Image), FBI_Width (Image), 0, Bpp* 8)
  Buf= Allocate (Bpp)
  Pitch= FBI_Pitch (Image)
  Start= FBI_Pixels (Image)+ (FBI_Height (Image)- 1)* Pitch
  Dst= FBI_Pixels (Buffer)
  For Y As Integer= 1 To FBI_Height (Buffer)
    Src= Start
    For X As Integer= 1 To FBI_Width (Buffer)
      MemCpy (Dst, Src, Bpp)
      Dst= Dst+ Bpp
      Src= Src- Pitch
    Next X
    Start= Start+ Bpp
  Next Y
  
  ImageDestroy (Image)
  Image= Buffer

  DeAllocate (Buf)
End Sub

/' --- TFBImage Properties ---------------------------------------- '/
Property TFBImage.FBI_Width (ByVal Image As PImage) As Integer
Dim Result As Integer

  ImageInfo (Image, Result)
  Property= Result
End Property

Property TFBImage.FBI_Height (ByVal Image As PImage) As Integer
Dim Result As Integer

  ImageInfo (Image,, Result)
  Property= Result
End Property

Property TFBImage.FBI_Bpp (ByVal Image As PImage) As Integer
Dim Result As Integer

  ImageInfo (Image,,, Result)
  Property= Result
End Property

Property TFBImage.FBI_Pitch (ByVal Image As PImage) As Integer
Dim Result As Integer

  ImageInfo (Image,,,, Result)
  Property= Result
End Property

Property TFBImage.FBI_Pixels (ByVal Image As PImage) As uByte Ptr
Dim Result As uByte Ptr

  ImageInfo (Image,,,,, Result)
  Property= Result
End Property

Property TFBImage.FBI_Size (ByVal Image As PImage) As Integer
Dim Result As Integer

  ImageInfo (Image,,,,,, Result)
  Property= Result
End Property
